@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2017 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2018 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -40,16 +40,22 @@
 
 #include "compiler.h"
 
-#include <stdio.h>
 #include <time.h>
 
 #include "nasmlib.h"
+#include "nctype.h"
 #include "strlist.h"
 #include "preproc.h"
 #include "insnsi.h"     /* For enum opcode */
 #include "directiv.h"   /* For enum directive */
+#include "labels.h"     /* For enum mangle_index, enum label_type */
 #include "opflags.h"
 #include "regs.h"
+#include "srcfile.h"
+#include "error.h"
+
+/* Program name for error messages etc. */
+extern const char *_progname;
 
 /* Time stamp for the official start of compilation */
 struct compile_time {
@@ -61,21 +67,9 @@ struct compile_time {
 };
 extern struct compile_time official_compile_time;
 
-#define NO_SEG -1L              /* null segment value */
+#define NO_SEG  INT32_C(-1)     /* null segment value */
 #define SEG_ABS 0x40000000L     /* mask for far-absolute segments */
 
-#ifndef FILENAME_MAX
-#define FILENAME_MAX 256
-#endif
-
-#ifndef PREFIX_MAX
-#define PREFIX_MAX 10
-#endif
-
-#ifndef POSTFIX_MAX
-#define POSTFIX_MAX 10
-#endif
-
 #define IDLEN_MAX 4096
 #define DECOLEN_MAX 32
 
@@ -100,6 +94,7 @@ struct ofmt;
 enum out_type {
     OUT_RAWDATA,    /* Plain bytes */
     OUT_RESERVE,    /* Reserved bytes (RESB et al) */
+    OUT_ZERODATA,   /* Initialized data, but all zero */
     OUT_ADDRESS,    /* An address (symbol value) */
     OUT_RELADDR,    /* A relative address */
     OUT_SEGMENT,    /* A segment number */
@@ -144,11 +139,6 @@ struct out_data {
     int64_t relbase;            /* Relative base for OUT_RELADDR */
 };
 
-/*
- * A label-lookup function.
- */
-typedef bool (*lfunc)(char *label, int32_t *segment, int64_t *offset);
-
 /*
  * And a label-definition function. The boolean parameter
  * `is_norm' states whether the label is a `normal' label (which
@@ -156,10 +146,7 @@ typedef bool (*lfunc)(char *label, int32_t *segment, int64_t *offset);
  * an EQU or a segment-base symbol, which shouldn't.
  */
 typedef void (*ldfunc)(char *label, int32_t segment, int64_t offset,
-                       char *special, bool is_norm, bool isextrn);
-
-void define_label(char *label, int32_t segment, int64_t offset,
-                  char *special, bool is_norm, bool isextrn);
+                       char *special, bool is_norm);
 
 /*
  * Token types returned by the scanner, in addition to ordinary
@@ -168,6 +155,7 @@ void define_label(char *label, int32_t segment, int64_t offset,
 enum token_type { /* token types, other than chars */
     TOKEN_INVALID = -1, /* a placeholder value */
     TOKEN_EOS = 0,      /* end of string */
+    TOKEN_QMARK = '?',
     TOKEN_EQ = '=',
     TOKEN_GT = '>',
     TOKEN_LT = '<',     /* aliases */
@@ -181,24 +169,29 @@ enum token_type { /* token types, other than chars */
     TOKEN_INSN,         /* instruction name */
     TOKEN_HERE,         /* $ */
     TOKEN_BASE,         /* $$ */
-    TOKEN_SPECIAL,      /* BYTE, WORD, DWORD, QWORD, FAR, NEAR, etc */
+    TOKEN_SIZE,		/* BYTE, WORD, DWORD, QWORD, etc */
+    TOKEN_SPECIAL,      /* REL, FAR, NEAR, STRICT, NOSPLIT, etc */
     TOKEN_PREFIX,       /* A32, O16, LOCK, REPNZ, TIMES, etc */
-    TOKEN_SHL,          /* << */
+    TOKEN_SHL,          /* << or <<< */
     TOKEN_SHR,          /* >> */
+    TOKEN_SAR,          /* >>> */
     TOKEN_SDIV,         /* // */
     TOKEN_SMOD,         /* %% */
     TOKEN_GE,           /* >= */
     TOKEN_LE,           /* <= */
     TOKEN_NE,           /* <> (!= is same as <>) */
+    TOKEN_LEG,          /* <=> */
     TOKEN_DBL_AND,      /* && */
     TOKEN_DBL_OR,       /* || */
     TOKEN_DBL_XOR,      /* ^^ */
     TOKEN_SEG,          /* SEG */
     TOKEN_WRT,          /* WRT */
-    TOKEN_FLOATIZE,     /* __floatX__ */
+    TOKEN_FLOATIZE,     /* __?floatX?__ */
     TOKEN_STRFUNC,      /* __utf16*__, __utf32*__ */
     TOKEN_IFUNC,        /* __ilog2*__ */
     TOKEN_DECORATOR,    /* decorators such as {...} */
+    TOKEN_MASM_PTR,     /* __?masm_ptr?__ for the masm package */
+    TOKEN_MASM_FLAT,    /* __?masm_flat?__ for the masm package */
     TOKEN_OPMASK        /* translated token for opmask registers */
 };
 
@@ -342,6 +335,13 @@ typedef expr *(*evalfunc)(scanner sc, void *scprivate,
 /*
  * preprocessors ought to look like this:
  */
+
+enum preproc_mode {
+    PP_NORMAL,                  /* Assembly */
+    PP_DEPS,                    /* Dependencies only */
+    PP_PREPROC                  /* Preprocessing only */
+};
+
 struct preproc_ops {
     /*
      * Called once at the very start of assembly.
@@ -353,7 +353,8 @@ struct preproc_ops {
      * of the pass, an error reporting function, an evaluator
      * function, and a listing generator to talk to.
      */
-    void (*reset)(char *file, int pass, StrList **deplist);
+    void (*reset)(const char *file, enum preproc_mode mode,
+                  struct strlist *deplist);
 
     /*
      * Called to fetch a line of preprocessed source. The line
@@ -362,8 +363,15 @@ struct preproc_ops {
      */
     char *(*getline)(void);
 
-    /* Called at the end of a pass */
-    void (*cleanup)(int pass);
+    /* Called at the end of each pass. */
+    void (*cleanup_pass)(void);
+
+    /*
+     * Called at the end of the assembly session,
+     * after cleanup_pass() has been called for the
+     * last pass.
+     */
+    void (*cleanup_session)(void);
 
     /* Additional macros specific to output format */
     void (*extra_stdmac)(macros_t *macros);
@@ -375,53 +383,27 @@ struct preproc_ops {
     /* Include file from command line */
     void (*pre_include)(char *fname);
 
+    /* Add a command from the command line */
+    void (*pre_command)(const char *what, char *str);
+
     /* Include path from command line */
-    void (*include_path)(char *path);
+    void (*include_path)(struct strlist *ipath);
 
     /* Unwind the macro stack when printing an error message */
-    void (*error_list_macros)(int severity);
+    void (*error_list_macros)(errflags severity);
+
+    /* Return true if an error message should be suppressed */
+    bool (*suppress_error)(errflags severity);
 };
 
 extern const struct preproc_ops nasmpp;
 extern const struct preproc_ops preproc_nop;
 
 /* List of dependency files */
-extern StrList *depend_list;
+extern struct strlist *depend_list;
 
-/*
- * Some lexical properties of the NASM source language, included
- * here because they are shared between the parser and preprocessor.
- */
-
-/*
- * isidstart matches any character that may start an identifier, and isidchar
- * matches any character that may appear at places other than the start of an
- * identifier. E.g. a period may only appear at the start of an identifier
- * (for local labels), whereas a number may appear anywhere *but* at the
- * start.
- * isbrcchar matches any character that may placed inside curly braces as a
- * decorator. E.g. {rn-sae}, {1to8}, {k1}{z}
- */
-
-#define isidstart(c) (nasm_isalpha(c)   ||  \
-                      (c) == '_'        ||  \
-                      (c) == '.'        ||  \
-                      (c) == '?'        ||  \
-                      (c) == '@')
-
-#define isidchar(c) (isidstart(c)       ||  \
-                     nasm_isdigit(c)    ||  \
-                     (c) == '$'         ||  \
-                     (c) == '#'         ||  \
-                     (c) == '~')
-
-#define isbrcchar(c) (isidchar(c)       ||  \
-                      (c) == '-')
-
-/* Ditto for numeric constants. */
-
-#define isnumstart(c)  (nasm_isdigit(c) || (c) == '$')
-#define isnumchar(c)   (nasm_isalnum(c) || (c) == '_')
+/* TASM mode changes some properties */
+extern bool tasm_compatible_mode;
 
 /*
  * inline function to skip past an identifier; returns the first character past
@@ -431,10 +413,10 @@ static inline char *nasm_skip_identifier(const char *str)
 {
     const char *p = str;
 
-    if (!isidstart(*p++)) {
+    if (!nasm_isidstart(*p++)) {
         p = NULL;
     } else {
-        while (isidchar(*p++))
+        while (nasm_isidchar(*p++))
             ;
     }
     return (char *)p;
@@ -480,6 +462,7 @@ enum ccode { /* condition code names */
 #define TFLAG_BRC_ANY   (TFLAG_BRC | TFLAG_BRC_OPT)
 #define TFLAG_BRDCAST   (1 << 2)    /* broadcasting decorator */
 #define TFLAG_WARN	(1 << 3)    /* warning only, treat as ID */
+#define TFLAG_DUP	(1 << 4)    /* valid ID but also has context-specific use */
 
 static inline uint8_t get_cond_opcode(enum ccode c)
 {
@@ -566,13 +549,6 @@ enum prefixes { /* instruction prefixes */
     PREFIX_ENUM_LIMIT
 };
 
-enum extop_type { /* extended operand types */
-    EOT_NOTHING,
-    EOT_DB_STRING,      /* Byte string */
-    EOT_DB_STRING_FREE, /* Byte string which should be nasm_free'd*/
-    EOT_DB_NUMBER       /* Integer */
-};
-
 enum ea_flags { /* special EA flags */
     EAF_BYTEOFFS    =  1,   /* force offset part to byte size */
     EAF_WORDOFFS    =  2,   /* force offset part to [d]word size */
@@ -613,15 +589,34 @@ typedef struct operand { /* operand to an instruction */
 #define OPFLAG_RELATIVE     8   /* operand is self-relative, e.g. [foo - $]
                                    where foo is not in the current segment */
 
+enum extop_type { /* extended operand types */
+    EOT_NOTHING = 0,
+    EOT_EXTOP,          /* Subexpression */
+    EOT_DB_STRING,      /* Byte string */
+    EOT_DB_FLOAT,       /* Floating-pointer number (special byte string) */
+    EOT_DB_STRING_FREE, /* Byte string which should be nasm_free'd*/
+    EOT_DB_NUMBER,      /* Integer */
+    EOT_DB_RESERVE      /* ? */
+};
+
 typedef struct extop { /* extended operand */
-    struct extop    *next;      /* linked list */
-    char            *stringval; /* if it's a string, then here it is */
-    size_t          stringlen;  /* ... and here's how long it is */
-    int64_t         offset;     /* ... it's given here ... */
-    int32_t         segment;    /* if it's a number/address, then... */
-    int32_t         wrt;        /* ... and here */
-    bool            relative;   /* self-relative expression */
-    enum extop_type type;       /* defined above */
+    struct extop    *next;       /* linked list */
+    union {
+        struct {                 /* text or byte string */
+            char    *data;
+            size_t   len;
+        } string;
+        struct {                 /* numeric expression */
+            int64_t  offset;     /* numeric value or address offset */
+            int32_t  segment;    /* address segment */
+            int32_t  wrt;        /* address wrt */
+            bool     relative;   /* self-relative expression */
+        } num;
+        struct extop *subexpr;   /* actual expressions */
+    } val;
+    size_t dup;                  /* duplicated? */
+    enum extop_type type;        /* defined above */
+    int elem;                    /* element size override, if any (bytes) */
 } extop;
 
 enum ea_type {
@@ -742,10 +737,11 @@ enum directive_result {
  * as part of the struct pragma.
  */
 struct pragma;
+typedef enum directive_result (*pragma_handler)(const struct pragma *);
 
 struct pragma_facility {
     const char *name;
-    enum directive_result (*handler)(const struct pragma *);
+    pragma_handler handler;
 };
 
 /*
@@ -765,6 +761,25 @@ struct pragma {
     enum directive opcode;     /* Operation as a D_ directives constant */
 };
 
+/*
+ * These are semi-arbitrary limits to keep the assembler from going
+ * into a black hole on certain kinds of bugs.  They can be overridden
+ * by command-line options or %pragma.
+ */
+enum nasm_limit {
+    LIMIT_PASSES,
+    LIMIT_STALLED,
+    LIMIT_MACRO_LEVELS,
+    LIMIT_MACRO_TOKENS,
+    LIMIT_MMACROS,
+    LIMIT_REP,
+    LIMIT_EVAL,
+    LIMIT_LINES
+};
+#define LIMIT_MAX LIMIT_LINES
+extern int64_t nasm_limit[LIMIT_MAX+1];
+extern enum directive_result  nasm_set_limit(const char *, const char *);
+
 /*
  * The data structure defining an output format driver, and the
  * interfaces to the functions therein.
@@ -781,10 +796,17 @@ struct ofmt {
      */
     const char *shortname;
 
+    /*
+     * Default output filename extension, or a null string
+     */
+    const char *extension;
+
     /*
      * Output format flags.
      */
-#define OFMT_TEXT   1		/* Text file format */
+#define OFMT_TEXT		1	/* Text file format */
+#define OFMT_KEEP_ADDR	2	/* Keep addr; no conversion to data */
+
     unsigned int flags;
 
     int maxbits;                /* Maximum segment bits supported */
@@ -802,7 +824,7 @@ struct ofmt {
     /*
      * This, if non-NULL, is a NULL-terminated list of `char *'s
      * pointing to extra standard macros supplied by the object
-     * format (e.g. a sensible initial default value of __SECT__,
+     * format (e.g. a sensible initial default value of __?SECT?__,
      * and user-level equivalents for any format-specific
      * directives).
      */
@@ -814,6 +836,11 @@ struct ofmt {
      */
     void (*init)(void);
 
+    /*
+     * This procedure is called at the start of each pass.
+     */
+    void (*reset)(void);
+
     /*
      * This is the modern output function, which gets passed
      * a struct out_data with much more information.  See the
@@ -885,7 +912,25 @@ struct ofmt {
      * the segment, by setting `*bits' to 16 or 32. Or, if it
      * doesn't wish to define a default, it can leave `bits' alone.
      */
-    int32_t (*section)(char *name, int pass, int *bits);
+    int32_t (*section)(char *name, int *bits);
+
+    /*
+     * This function is called when a label is defined
+     * in the source code. It is allowed to change the section
+     * number as a result, but not the bits value.
+     * This is *only* called if the symbol defined is at the
+     * current offset, i.e. "foo:" or "foo equ $".
+     * The offset isn't passed; and may not be stable at this point.
+     * The subsection number is a field available for use by the
+     * backend. It is initialized to NO_SEG.
+     *
+     * If "copyoffset" is set by the backend then the offset is
+     * copied from the previous segment, otherwise the new segment
+     * is treated as a new segment the normal way.
+     */
+    int32_t (*herelabel)(const char *name, enum label_type type,
+                         int32_t seg, int32_t *subsection,
+                         bool *copyoffset);
 
     /*
      * This procedure is called to modify section alignment,
@@ -912,8 +957,7 @@ struct ofmt {
      * This procedure is called to allow the output driver to
      * process its own specific directives. When called, it has the
      * directive word in `directive' and the parameter string in
-     * `value'. It is called in both assembly passes, and `pass'
-     * will be either 1 or 2.
+     * `value'.
      *
      * The following values are (currently) possible for
      * directive_result:
@@ -925,26 +969,7 @@ struct ofmt {
      *				  "invalid parameter to [*] directive"
      */
     enum directive_result
-    (*directive)(enum directive directive, char *value, int pass);
-
-    /*
-     * This procedure is called before anything else - even before
-     * the "init" routine - and is passed the name of the input
-     * file from which this output file is being generated. It
-     * should return its preferred name for the output file in
-     * `outname', if outname[0] is not '\0', and do nothing to
-     * `outname' otherwise. Since it is called before the driver is
-     * properly initialized, it has to be passed its error handler
-     * separately.
-     *
-     * This procedure may also take its own copy of the input file
-     * name for use in writing the output file: it is _guaranteed_
-     * that it will be called before the "init" routine.
-     *
-     * The parameter `outname' points to an area of storage
-     * guaranteed to be at least FILENAME_MAX in size.
-     */
-    void (*filename)(char *inname, char *outname);
+    (*directive)(enum directive directive, char *value);
 
     /*
      * This procedure is called after assembly finishes, to allow
@@ -968,7 +993,6 @@ struct ofmt {
  */
 struct ofmt_alias {
     const char  *shortname;
-    const char  *fullname;
     const struct ofmt *ofmt;
 };
 
@@ -1079,25 +1103,40 @@ extern const struct dfmt *dfmt;
 
 #define TYS_ELEMENTS(x) ((x) << 8)
 
+/* Sizes corresponding to various tokens */
+enum byte_sizes {
+    SIZE_BYTE	=  1,
+    SIZE_WORD	=  2,
+    SIZE_DWORD	=  4,
+    SIZE_QWORD	=  8,
+    SIZE_TWORD  = 10,
+    SIZE_OWORD  = 16,
+    SIZE_YWORD  = 32,
+    SIZE_ZWORD  = 64
+};
+
 enum special_tokens {
-    SPECIAL_ENUM_START  = PREFIX_ENUM_LIMIT,
-    S_ABS               = SPECIAL_ENUM_START,
-    S_BYTE,
+    SIZE_ENUM_START     = PREFIX_ENUM_LIMIT,
+    S_BYTE              = SIZE_ENUM_START,
+    S_WORD,
     S_DWORD,
+    S_QWORD,
+    S_TWORD,
+    S_OWORD,
+    S_YWORD,
+    S_ZWORD,
+    SIZE_ENUM_LIMIT,
+
+    SPECIAL_ENUM_START  = SIZE_ENUM_LIMIT,
+    S_ABS		= SPECIAL_ENUM_START,
     S_FAR,
     S_LONG,
     S_NEAR,
     S_NOSPLIT,
-    S_OWORD,
-    S_QWORD,
     S_REL,
     S_SHORT,
     S_STRICT,
     S_TO,
-    S_TWORD,
-    S_WORD,
-    S_YWORD,
-    S_ZWORD,
     SPECIAL_ENUM_LIMIT
 };
 
@@ -1227,20 +1266,82 @@ enum decorator_tokens {
  */
 
 /*
- * This declaration passes the "pass" number to all other modules
- * "pass0" assumes the values: 0, 0, ..., 0, 1, 2
- * where 0 = optimizing pass
- *       1 = pass 1
- *       2 = pass 2
+ * flag to disable optimizations selectively
+ * this is useful to turn-off certain optimizations
  */
+enum optimization_disable_flag {
+    OPTIM_ALL_ENABLED       = 0,
+    OPTIM_DISABLE_JMP_MATCH = 1
+};
 
-extern int pass0;
-extern int passn;               /* Actual pass number */
+struct optimization {
+    int level;
+    int flag;
+};
 
-extern bool tasm_compatible_mode;
-extern int optimizing;
+/*
+ * Various types of compiler passes we may execute.
+ */
+enum pass_type {
+    PASS_INIT,            /* Initialization, not doing anything yet */
+    PASS_FIRST,           /* The very first pass over the code */
+    PASS_OPT,             /* Optimization pass */
+    PASS_STAB,            /* Stabilization pass (original pass 1) */
+    PASS_FINAL            /* Code generation pass (original pass 2) */
+};
+extern const char * const _pass_types[];
+extern enum pass_type _pass_type;
+static inline enum pass_type pass_type(void)
+{
+    return _pass_type;
+}
+static inline const char *pass_type_name(void)
+{
+    return _pass_types[_pass_type];
+}
+/* True during initialization, no code read yet */
+static inline bool not_started(void)
+{
+    return pass_type() == PASS_INIT;
+}
+/* True for the initial pass and setup (old "pass2 < 2") */
+static inline bool pass_first(void)
+{
+    return pass_type() <= PASS_FIRST;
+}
+/* At this point we better have stable definitions */
+static inline bool pass_stable(void)
+{
+    return pass_type() >= PASS_STAB;
+}
+/* True for the code generation pass only, (old "pass1 >= 2") */
+static inline bool pass_final(void)
+{
+    return pass_type() >= PASS_FINAL;
+}
+
+/*
+ * The actual pass number. 0 is used during initialization, the very
+ * first pass is 1, and then it is simply increasing numbers until we are
+ * done.
+ */
+extern int64_t _passn;           /* Actual pass number */
+static inline int64_t pass_count(void)
+{
+    return _passn;
+}
+
+extern struct optimization optimizing;
 extern int globalbits;          /* 16, 32 or 64-bit mode */
 extern int globalrel;           /* default to relative addressing? */
 extern int globalbnd;           /* default to using bnd prefix? */
 
-#endif
+extern const char *inname;	/* primary input filename */
+extern const char *outname;     /* output filename */
+
+/*
+ * Switch to a different segment and return the current offset
+ */
+int64_t switch_segment(int32_t segment);
+
+#endif  /* NASM_NASM_H */
