@@ -62,7 +62,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <string.h>
 #include <time.h>
 
 /* functions supported:
@@ -233,6 +232,7 @@ int main(int argc, char **argv)
             fprintf(stderr, "rdflib: required parameter missing\n");
             exit(1);
         }
+        break;
     case 't':
         fp = fopen(argv[2], "rb");
         if (!fp) {
@@ -317,6 +317,8 @@ int main(int argc, char **argv)
 
     case 'r':                  /* replace module */
         argc--;
+        /* fall through */
+
     case 'd':                  /* delete module */
         if (argc < 4) {
             fprintf(stderr, "rdflib: required parameter missing\n");
@@ -371,7 +373,7 @@ int main(int argc, char **argv)
             /* check against desired name */
             if (!strcmp(buf, argv[3])) {
                 if (fread(p = rdbuf, 1, sizeof(rdbuf), fptmp) < 10) {
-                    nasm_fatal(0, "short read on input");
+                    nasm_fatal("short read on input");
                 }
                 l = *(int32_t *)(p + 6);
                 fseek(fptmp, l, SEEK_CUR);
